/*
 *  Copyright (c) 2001 Dan Gudmundsson
 *  See the file "license.terms" for information on usage and redistribution
 *  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 * 
 *     $Id: esdl_wrapper.c,v 1.16 2003/05/25 05:54:11 bjorng Exp $
 */
/*    
   Command Handler
   Dispatch the work on the functions   
*/

#include <stdlib.h>

#include "esdl.h"
#include "esdl_gl.h"
#include "esdl_glu.h"
#include "esdl_util.h"
#ifndef APIENTRY
# define APIENTRY
#endif
#define ESDL_DEFINE_EXTS
#include "esdl_glext.h"

typedef struct sdl_code_fn_def {
   int op;
   char *str;
   sdl_fun fn;
} sdl_code_fn;

static sdl_code_fn code_fns[] = {
   { SDL_InitFunc, "SDL_InitFunc", es_init },
   { SDL_QuitFunc, "SDL_QuitFunc", es_quit },
   { SDL_GetErrorFunc, "SDL_GetErrorFunc", es_getError },
   { SDL_SetVideoModeFunc, "SDL_SetVideoModeFunc",  es_setVideoMode },      
   { SDL_VideoDriverNameFunc, "SDL_VideoDriverNameFunc",  es_videoDriverName },      
   { SDL_GetVideoSurfaceFunc, "SDL_GetVideoSurfaceFunc",  es_getVideoSurface },
   { SDL_GetVideoInfoFunc, "SDL_GetVideoInfoFunc",     es_getVideoInfo },
   { SDL_VideoModeOKFunc, "SDL_VideoModeOKFunc",      es_videoModeOK },  
   { SDL_ListModesFunc, "SDL_ListModesFunc",        es_listModes },
   { SDL_SetColorKeyFunc, "SDL_SetColorKeyFunc",      es_setColorKey },
   { SDL_DisplayFormatFunc, "SDL_DisplayFormatFunc",    es_displayFormat },
   { SDL_BlitSurfaceFunc, "SDL_BlitSurfaceFunc",      es_blitSurface },
   { SDL_FillRectFunc, "SDL_FillRectFunc",         es_fillRect },
   { SDL_UpdateRectsFunc, "SDL_UpdateRectsFunc",      es_updateRects },
   { SDL_FlipFunc, "SDL_FlipFunc",             es_flip },
   { SDL_SetColorsFunc, "SDL_SetColorsFunc",        es_setColors },
   { SDL_MapRGBFunc, "SDL_MapRGBFunc",           es_mapRGB },
   { SDL_GetRGBFunc, "SDL_GetRGBFunc",           es_getRGB },
   { SDL_CreateRGBSurfaceFunc, "SDL_CreateRGBSurfaceFunc", es_createRGBSurface },
   { SDL_CreateRGBSurfaceFromFunc, "SDL_CreateRGBSurfaceFromFunc",  es_createRGBSurfaceFrom },
   { SDL_FreeSurfaceFunc, "SDL_FreeSurfaceFunc",      es_freeSurface },
   { SDL_LockSurfaceFunc, "SDL_LockSurfaceFunc",      es_lockSurface },
   { SDL_UnlockSurfaceFunc, "SDL_UnlockSurfaceFunc",    es_unlockSurface },
   { SDL_LoadBMPFunc, "SDL_LoadBMPFunc",    es_loadBMP },
   { SDL_SaveBMPFunc, "SDL_SaveBMPFunc",    es_saveBMP }, 
   { SDL_SetAlphaFunc, "SDL_SetAlphaFunc",    es_setAlpha }, 
   { SDL_SetClippingFunc, "SDL_SetClippingFunc",    es_setClipping },

   { SDL_SetGammaFunc,     "SDL_SetGamma",     es_setGamma },
   { SDL_SetGammaRampFunc, "SDL_SetGammaRamp", es_setGammaRamp },
   { SDL_GetGammaRampFunc, "SDL_GetGamma",     es_getGammaRamp },

   { SDL_WM_SetCaptionFunc, "SDL_WM_SetCaptionFunc", es_wm_setCaption },
   { SDL_WM_GetCaptionFunc, "SDL_WM_GetCaptionFunc", es_wm_getCaption },
   { SDL_WM_SetIconFunc, "SDL_WM_SetIconFunc",    es_wm_setIcon },
   { SDL_WM_IconifyWindowFunc, "SDL_WM_IconifyWindowFunc",    es_wm_iconifyWindow },
   { SDL_WM_ToggleFullScreenFunc, "SDL_WM_ToggleFullScreenFunc",    es_wm_toggleFullScreen },
   { SDL_WM_GrabInputFunc, "SDL_WM_GrabInputFunc",    es_wm_grabInput },
   { SDL_WM_GetInfoFunc, "SDL_WM_GetInfoFunc",     es_wm_getInfo },
   { SDL_WM_IsMaximizedFunc, "SDL_WM_IsMaximizedFunc",     es_wm_isMaximized },

   /** EVENT FUNCTIONS */
   { SDL_PumpEventsFunc, "SDL_PumpEventsFunc",  es_pumpEvents },
   { SDL_PeepEventsFunc, "SDL_PeepEventsFunc",    es_peepEvents },
   { SDL_PollEventFunc, "SDL_PollEventFunc",    es_pollEvent },
   { SDL_WaitEventFunc, "SDL_WaitEventFunc",    es_waitEvent },
   { SDL_EventStateFunc, "SDL_EventStateFunc",    es_eventState },
   { SDL_GetAppStateFunc, "SDL_GetAppStateFunc", es_getAppState },
   { SDL_GetModStateFunc, "SDL_GetModStateFunc", es_getModState },
   { SDL_SetModStateFunc, "SDL_SetModStateFunc", es_setModState },
   { SDL_GetKeyStateFunc, "SDL_GetKeyStateFunc", es_getKeyState },
   { SDL_EnableUNICODEFunc, "SDL_EnableUNICODEFunc", es_enableUNICODE },
   { SDL_GetKeyNameFunc, "SDL_GetKeyNameFunc",    es_getKeyName },
   { SDL_EnableKeyRepeatFunc, "SDL_EnableKeyRepeatFunc",    es_enableKeyRepeat },

   /* Mouse Functions */
   { SDL_GetMouseStateFunc, "SDL_GetMouseStateFunc",  es_getMouseState },
   { SDL_GetRelativeMouseStateFunc, "SDL_GetRelativeMouseStateFunc",    es_getRelativeMouseState },
   { SDL_WarpMouseFunc, "SDL_WarpMouseFunc",    es_warpMouse },
   { SDL_CreateCursorFunc, "SDL_CreateCursorFunc",    es_createCursor },
   { SDL_SetCursorFunc, "SDL_SetCursorFunc",    es_setCursor },   
   { SDL_GetCursorFunc, "SDL_GetCursorFunc",    es_getCursor },
   { SDL_FreeCursorFunc, "SDL_FreeCursorFunc",    es_freeCursor },
   { SDL_ShowCursorFunc, "SDL_ShowCursorFunc",    es_showCursor },

   /* Joystick functions */
   { SDL_NumJoysticksFunc, "SDL_NumJoysticksFunc", es_numJoysticks },
   { SDL_JoystickNameFunc, "SDL_JoystickNameFunc", es_joystick_name },
   { SDL_JoystickOpenFunc, "SDL_JoystickOpenFunc", es_joystick_open },
   { SDL_JoystickOpenedFunc, "SDL_JoystickOpenedFunc", es_joystick_opened },
   { SDL_JoystickIndexFunc, "SDL_JoystickIndexFunc", es_joystick_index },
   { SDL_JoystickNumAxesFunc, "SDL_JoystickNumAxesFunc", es_joystick_numAxes },
   { SDL_JoystickNumBallsFunc, "SDL_JoystickNumBallsFunc", es_joystick_numBalls },
   { SDL_JoystickNumHatsFunc, "SDL_JoystickNumHatsFunc", es_joystick_numHats },
   { SDL_JoystickNumButtonsFunc, "SDL_JoystickNumButtonsFunc", es_joystick_numButtons },
   { SDL_JoystickUpdateFunc, "SDL_JoystickUpdateFunc", es_joystick_update },
   { SDL_JoystickEventStateFunc, "SDL_JoystickEventStateFunc", es_joystick_eventState },
   { SDL_JoystickGetAxisFunc, "SDL_JoystickGetAxisFunc", es_joystick_getAxis },
   { SDL_JoystickGetHatFunc, "SDL_JoystickGetHatFunc", es_joystick_getHat },
   { SDL_JoystickGetButtonFunc, "SDL_JoystickGetButtonFunc", es_joystick_getButton },
   { SDL_JoystickGetBallFunc, "SDL_JoystickGetBallFunc", es_joystick_getBall },
   { SDL_JoystickCloseFunc, "SDL_JoystickCloseFunc", es_joystick_close },

   /* SDL open gl support funcs */
   { SDL_GL_SetAttributeFunc, "SDL_GL_SetAttributeFunc", es_gl_setAttribute },
   { SDL_GL_GetAttributeFunc, "SDL_GL_GetAttributeFunc",    es_gl_getAttribute },
   { SDL_GL_SwapBuffersFunc, "SDL_GL_SwapBuffersFunc",    es_gl_swapBuffers },

   /* Erl sdl functions */
   { ESDL_getSurfaceFunc, "ESDL_getSurfaceFunc",  es_getSurface },
   { ESDL_getPixelFormatFunc, "ESDL_getPixelFormatFunc",    es_getPixelFormat },
   { ESDL_getPaletteFunc, "ESDL_getPaletteFunc",    es_getPalette },
   { ESDL_getPixelsFunc, "ESDL_getPixelsFunc",    es_getPixels },
   { SDL_UTIL_copySdlImage2GLArrayFunc, "SDL_UTIL_copySdlImage2GLArrayFunc",    copySdlImage2GLArray },
   { SDL_UTIL_DebugFunc, "SDL_UTIL_DebugFunc", sdl_util_debug},

   /* SDL AUDIO functions */
   { SDL_AudioDriverNameFunc, "SDL_AudioDriverNameFunc", es_audioDriverName },
   { SDL_OpenAudioFunc, "SDL_OpenAudioFunc", es_openAudio },
   { SDL_GetAudioStatusFunc, "SDL_GetAudioStatusFunc",    es_getAudioStatus },
   { SDL_PauseAudioFunc, "SDL_PauseAudioFunc",    es_pauseAudio },
   { SDL_LoadWAVFunc, "SDL_LoadWAVFunc",    es_loadWAV },

   { SDL_FreeWAVFunc, "SDL_FreeWAVFunc",  es_freeWAV },
   { SDL_BuildAudioCVTFunc, "SDL_BuildAudioCVTFunc",    es_buildAudioCVT },
   { SDL_ConvertAudioFunc, "SDL_ConvertAudioFunc",    es_convertAudio },
   { SDL_MixAudioFunc, "SDL_MixAudioFunc",    es_mixAudio },
   { SDL_LockAudioFunc, "SDL_LockAudioFunc",    es_lockAudio },
   { SDL_UnlockAudioFunc, "SDL_UnlockAudioFunc",    es_unlockAudio },
   { SDL_CloseAudioFunc, "SDL_CloseAudioFunc",    es_closeAudio },
   { play_audioFunc, "play_audioFunc",    play_audio },

   /* OPENGL 1.1 */
   { glAccumFunc,  "glAccumFunc", egl_accum },
   { glAlphaFuncFunc,  "glAlphaFuncFunc", egl_alphaFunc },
   { glAreTexturesResidentFunc,  "glAreTexturesResidentFunc", egl_areTexturesResident },
   { glArrayElementFunc,  "glArrayElementFunc", egl_arrayElement },
   { glBeginFunc,  "glBeginFunc", egl_begin },
   { glBindTextureFunc,  "glBindTextureFunc", egl_bindTexture },
   { glBitmapFunc,  "glBitmapFunc", egl_bitmap },
   { glBlendFuncFunc,  "glBlendFuncFunc", egl_blendFunc },
   { glCallListFunc,  "glCallListFunc", egl_callList },
   { glCallListsFunc,  "glCallListsFunc", egl_callLists },
   { glClearFunc,  "glClearFunc", egl_clear },
   { glClearAccumFunc,  "glClearAccumFunc", egl_clearAccum },
   { glClearColorFunc,  "glClearColorFunc", egl_clearColor },
   { glClearDepthFunc,  "glClearDepthFunc", egl_clearDepth },
   { glClearIndexFunc,  "glClearIndexFunc", egl_clearIndex },
   { glClearStencilFunc,  "glClearStencilFunc", egl_clearStencil },
   { glClipPlaneFunc,  "glClipPlaneFunc", egl_clipPlane },
   { glColor3bvFunc,  "glColor3bvFunc", egl_color3bv },
   { glColor3dvFunc,  "glColor3dvFunc", egl_color3dv },
   { glColor3fvFunc,  "glColor3fvFunc", egl_color3fv },
   { glColor3ivFunc,  "glColor3ivFunc", egl_color3iv },
   { glColor3svFunc,  "glColor3svFunc", egl_color3sv },
   { glColor3ubvFunc,  "glColor3ubvFunc", egl_color3ubv },
   { glColor3uivFunc,  "glColor3uivFunc", egl_color3uiv },
   { glColor3usvFunc,  "glColor3usvFunc", egl_color3usv },
   { glColor4bvFunc,  "glColor4bvFunc", egl_color4bv },
   { glColor4dvFunc,  "glColor4dvFunc", egl_color4dv },
   { glColor4fvFunc,  "glColor4fvFunc", egl_color4fv },
   { glColor4ivFunc,  "glColor4ivFunc", egl_color4iv },
   { glColor4svFunc,  "glColor4svFunc", egl_color4sv },
   { glColor4ubvFunc,  "glColor4ubvFunc", egl_color4ubv },
   { glColor4uivFunc,  "glColor4uivFunc", egl_color4uiv },
   { glColor4usvFunc,  "glColor4usvFunc", egl_color4usv },
   { glColorMaskFunc,  "glColorMaskFunc", egl_colorMask },
   { glColorMaterialFunc,  "glColorMaterialFunc", egl_colorMaterial },
   { glColorPointerFunc,  "glColorPointerFunc", egl_colorPointer },
   { glCopyPixelsFunc,  "glCopyPixelsFunc", egl_copyPixels },
   { glCopyTexImage1DFunc,  "glCopyTexImage1DFunc", egl_copyTexImage1D },
   { glCopyTexImage2DFunc,  "glCopyTexImage2DFunc", egl_copyTexImage2D },
   { glCopyTexSubImage1DFunc,  "glCopyTexSubImage1DFunc", egl_copyTexSubImage1D },
   { glCopyTexSubImage2DFunc,  "glCopyTexSubImage2DFunc", egl_copyTexSubImage2D },
   { glCullFaceFunc,  "glCullFaceFunc", egl_cullFace },
   { glDeleteListsFunc,  "glDeleteListsFunc", egl_deleteLists },
   { glDeleteTexturesFunc,  "glDeleteTexturesFunc", egl_deleteTextures },
   { glDepthFuncFunc,  "glDepthFuncFunc", egl_depthFunc },
   { glDepthMaskFunc,  "glDepthMaskFunc", egl_depthMask },
   { glDepthRangeFunc,  "glDepthRangeFunc", egl_depthRange },
   { glDisableFunc,  "glDisableFunc", egl_disable },
   { glDisableClientStateFunc,  "glDisableClientStateFunc", egl_disableClientState },
   { glDrawArraysFunc,  "glDrawArraysFunc", egl_drawArrays },
   { glDrawBufferFunc,  "glDrawBufferFunc", egl_drawBuffer },
   { glDrawElementsFunc,  "glDrawElementsFunc", egl_drawElements },
   { glDrawPixelsFunc,  "glDrawPixelsFunc", egl_drawPixels },
   { glEdgeFlagFunc,  "glEdgeFlagFunc", egl_edgeFlag },
   { glEdgeFlagPointerFunc,  "glEdgeFlagPointerFunc", egl_edgeFlagPointer },
   { glEdgeFlagvFunc,  "glEdgeFlagvFunc", egl_edgeFlagv },
   { glEnableFunc,  "glEnableFunc", egl_enable },
   { glEnableClientStateFunc,  "glEnableClientStateFunc", egl_enableClientState },
   { glEndFunc,  "glEndFunc", egl_end },
   { glEndListFunc,  "glEndListFunc", egl_endList },
   { glEvalCoord1dvFunc,  "glEvalCoord1dvFunc", egl_evalCoord1dv },
   { glEvalCoord1fvFunc,  "glEvalCoord1fvFunc", egl_evalCoord1fv },
   { glEvalCoord2dvFunc,  "glEvalCoord2dvFunc", egl_evalCoord2dv },
   { glEvalCoord2fvFunc,  "glEvalCoord2fvFunc", egl_evalCoord2fv },
   { glEvalMesh1Func,  "glEvalMesh1Func", egl_evalMesh1 },
   { glEvalMesh2Func,  "glEvalMesh2Func", egl_evalMesh2 },
   { glEvalPoint1Func,  "glEvalPoint1Func", egl_evalPoint1 },
   { glEvalPoint2Func,  "glEvalPoint2Func", egl_evalPoint2 },
   { glFeedbackBufferFunc,  "glFeedbackBufferFunc", egl_feedbackBuffer },
   { glFinishFunc,  "glFinishFunc", egl_finish },
   { glFlushFunc,  "glFlushFunc", egl_flush },
   { glFogfFunc,  "glFogfFunc", egl_fogf },
   { glFogfvFunc,  "glFogfvFunc", egl_fogfv },
   { glFogiFunc,  "glFogiFunc", egl_fogi },
   { glFogivFunc,  "glFogivFunc", egl_fogiv },
   { glFrontFaceFunc,  "glFrontFaceFunc", egl_frontFace },
   { glFrustumFunc,  "glFrustumFunc", egl_frustum },
   { glGenListsFunc,  "glGenListsFunc", egl_genLists },
   { glGenTexturesFunc,  "glGenTexturesFunc", egl_genTextures },
   { glGetBooleanvFunc,  "glGetBooleanvFunc", egl_getBooleanv },
   { glGetClipPlaneFunc,  "glGetClipPlaneFunc", egl_getClipPlane },
   { glGetDoublevFunc,  "glGetDoublevFunc", egl_getDoublev },
   { glGetErrorFunc,  "glGetErrorFunc", egl_getError },
   { glGetFloatvFunc,  "glGetFloatvFunc", egl_getFloatv },
   { glGetIntegervFunc,  "glGetIntegervFunc", egl_getIntegerv },
   { glGetLightfvFunc,  "glGetLightfvFunc", egl_getLightfv },
   { glGetLightivFunc,  "glGetLightivFunc", egl_getLightiv },
   { glGetMapdvFunc,  "glGetMapdvFunc", egl_getMapdv },
   { glGetMapfvFunc,  "glGetMapfvFunc", egl_getMapfv },
   { glGetMapivFunc,  "glGetMapivFunc", egl_getMapiv },
   { glGetMaterialfvFunc,  "glGetMaterialfvFunc", egl_getMaterialfv },
   { glGetMaterialivFunc,  "glGetMaterialivFunc", egl_getMaterialiv },
   { glGetPixelMapfvFunc,  "glGetPixelMapfvFunc", egl_getPixelMapfv },
   { glGetPixelMapuivFunc,  "glGetPixelMapuivFunc", egl_getPixelMapuiv },
   { glGetPixelMapusvFunc,  "glGetPixelMapusvFunc", egl_getPixelMapusv },
   { glGetPointervFunc,  "glGetPointervFunc", egl_getPointerv },
   { glGetPolygonStippleFunc,  "glGetPolygonStippleFunc", egl_getPolygonStipple },
   { glGetStringFunc,  "glGetStringFunc", egl_getString },
   { glGetTexEnvfvFunc,  "glGetTexEnvfvFunc", egl_getTexEnvfv },
   { glGetTexEnvivFunc,  "glGetTexEnvivFunc", egl_getTexEnviv },
   { glGetTexGendvFunc,  "glGetTexGendvFunc", egl_getTexGendv },
   { glGetTexGenfvFunc,  "glGetTexGenfvFunc", egl_getTexGenfv },
   { glGetTexGenivFunc,  "glGetTexGenivFunc", egl_getTexGeniv },
   { glGetTexImageFunc,  "glGetTexImageFunc", egl_getTexImage },
   { glGetTexLevelParameterfvFunc,  "glGetTexLevelParameterfvFunc", egl_getTexLevelParameterfv },
   { glGetTexLevelParameterivFunc,  "glGetTexLevelParameterivFunc", egl_getTexLevelParameteriv },
   { glGetTexParameterfvFunc,  "glGetTexParameterfvFunc", egl_getTexParameterfv },
   { glGetTexParameterivFunc,  "glGetTexParameterivFunc", egl_getTexParameteriv },
   { glHintFunc,  "glHintFunc", egl_hint },
   { glIndexMaskFunc,  "glIndexMaskFunc", egl_indexMask },
   { glIndexPointerFunc,  "glIndexPointerFunc", egl_indexPointer },
   { glIndexdFunc,  "glIndexdFunc", egl_indexd },
   { glIndexdvFunc,  "glIndexdvFunc", egl_indexdv },
   { glIndexfFunc,  "glIndexfFunc", egl_indexf },
   { glIndexfvFunc,  "glIndexfvFunc", egl_indexfv },
   { glIndexiFunc,  "glIndexiFunc", egl_indexi },
   { glIndexivFunc,  "glIndexivFunc", egl_indexiv },
   { glIndexsFunc,  "glIndexsFunc", egl_indexs },
   { glIndexsvFunc,  "glIndexsvFunc", egl_indexsv },
   { glIndexubFunc,  "glIndexubFunc", egl_indexub },
   { glIndexubvFunc,  "glIndexubvFunc", egl_indexubv },
   { glInitNamesFunc,  "glInitNamesFunc", egl_initNames },
   { glInterleavedArraysFunc,  "glInterleavedArraysFunc", egl_interleavedArrays },
   { glIsEnabledFunc,  "glIsEnabledFunc", egl_isEnabled },
   { glIsListFunc,  "glIsListFunc", egl_isList },
   { glIsTextureFunc,  "glIsTextureFunc", egl_isTexture },
   { glLightModelfFunc,  "glLightModelfFunc", egl_lightModelf },
   { glLightModelfvFunc,  "glLightModelfvFunc", egl_lightModelfv },
   { glLightModeliFunc,  "glLightModeliFunc", egl_lightModeli },
   { glLightModelivFunc,  "glLightModelivFunc", egl_lightModeliv },
   { glLightfFunc,  "glLightfFunc", egl_lightf },
   { glLightfvFunc,  "glLightfvFunc", egl_lightfv },
   { glLightiFunc,  "glLightiFunc", egl_lighti },
   { glLightivFunc,  "glLightivFunc", egl_lightiv },
   { glLineStippleFunc,  "glLineStippleFunc", egl_lineStipple },
   { glLineWidthFunc,  "glLineWidthFunc", egl_lineWidth },
   { glListBaseFunc,  "glListBaseFunc", egl_listBase },
   { glLoadIdentityFunc,  "glLoadIdentityFunc", egl_loadIdentity },
   { glLoadMatrixdFunc,  "glLoadMatrixdFunc", egl_loadMatrixd },
   { glLoadMatrixfFunc,  "glLoadMatrixfFunc", egl_loadMatrixf },
   { glLoadNameFunc,  "glLoadNameFunc", egl_loadName },
   { glLogicOpFunc,  "glLogicOpFunc", egl_logicOp },
   { glMap1dFunc,  "glMap1dFunc", egl_map1d },
   { glMap1fFunc,  "glMap1fFunc", egl_map1f },
   { glMap2dFunc,  "glMap2dFunc", egl_map2d },
   { glMap2fFunc,  "glMap2fFunc", egl_map2f },
   { glMapGrid1dFunc,  "glMapGrid1dFunc", egl_mapGrid1d },
   { glMapGrid1fFunc,  "glMapGrid1fFunc", egl_mapGrid1f },
   { glMapGrid2dFunc,  "glMapGrid2dFunc", egl_mapGrid2d },
   { glMapGrid2fFunc,  "glMapGrid2fFunc", egl_mapGrid2f },
   { glMaterialfFunc,  "glMaterialfFunc", egl_materialf },
   { glMaterialfvFunc,  "glMaterialfvFunc", egl_materialfv },
   { glMaterialiFunc,  "glMaterialiFunc", egl_materiali },
   { glMaterialivFunc,  "glMaterialivFunc", egl_materialiv },
   { glMatrixModeFunc,  "glMatrixModeFunc", egl_matrixMode },
   { glMultMatrixdFunc,  "glMultMatrixdFunc", egl_multMatrixd },
   { glMultMatrixfFunc,  "glMultMatrixfFunc", egl_multMatrixf },
   { glNewListFunc,  "glNewListFunc", egl_newList },
   { glNormal3bvFunc,  "glNormal3bvFunc", egl_normal3bv },
   { glNormal3dvFunc,  "glNormal3dvFunc", egl_normal3dv },
   { glNormal3fvFunc,  "glNormal3fvFunc", egl_normal3fv },
   { glNormal3ivFunc,  "glNormal3ivFunc", egl_normal3iv },
   { glNormal3svFunc,  "glNormal3svFunc", egl_normal3sv },
   { glNormalPointerFunc,  "glNormalPointerFunc", egl_normalPointer },
   { glOrthoFunc,  "glOrthoFunc", egl_ortho },
   { glPassThroughFunc,  "glPassThroughFunc", egl_passThrough },
   { glPixelMapfvFunc,  "glPixelMapfvFunc", egl_pixelMapfv },
   { glPixelMapuivFunc,  "glPixelMapuivFunc", egl_pixelMapuiv },
   { glPixelMapusvFunc,  "glPixelMapusvFunc", egl_pixelMapusv },
   { glPixelStorefFunc,  "glPixelStorefFunc", egl_pixelStoref },
   { glPixelStoreiFunc,  "glPixelStoreiFunc", egl_pixelStorei },
   { glPixelTransferfFunc,  "glPixelTransferfFunc", egl_pixelTransferf },
   { glPixelTransferiFunc,  "glPixelTransferiFunc", egl_pixelTransferi },
   { glPixelZoomFunc,  "glPixelZoomFunc", egl_pixelZoom },
   { glPointSizeFunc,  "glPointSizeFunc", egl_pointSize },
   { glPolygonModeFunc,  "glPolygonModeFunc", egl_polygonMode },
   { glPolygonOffsetFunc,  "glPolygonOffsetFunc", egl_polygonOffset },
   { glPolygonStippleFunc,  "glPolygonStippleFunc", egl_polygonStipple },
   { glPopAttribFunc,  "glPopAttribFunc", egl_popAttrib },
   { glPopClientAttribFunc,  "glPopClientAttribFunc", egl_popClientAttrib },
   { glPopMatrixFunc,  "glPopMatrixFunc", egl_popMatrix },
   { glPopNameFunc,  "glPopNameFunc", egl_popName },
   { glPrioritizeTexturesFunc,  "glPrioritizeTexturesFunc", egl_prioritizeTextures },
   { glPushAttribFunc,  "glPushAttribFunc", egl_pushAttrib },
   { glPushClientAttribFunc,  "glPushClientAttribFunc", egl_pushClientAttrib },
   { glPushMatrixFunc,  "glPushMatrixFunc", egl_pushMatrix },
   { glPushNameFunc,  "glPushNameFunc", egl_pushName },
   { glRasterPos2dvFunc,  "glRasterPos2dvFunc", egl_rasterPos2dv },
   { glRasterPos2fvFunc,  "glRasterPos2fvFunc", egl_rasterPos2fv },
   { glRasterPos2ivFunc,  "glRasterPos2ivFunc", egl_rasterPos2iv },
   { glRasterPos2svFunc,  "glRasterPos2svFunc", egl_rasterPos2sv },
   { glRasterPos3dvFunc,  "glRasterPos3dvFunc", egl_rasterPos3dv },
   { glRasterPos3fvFunc,  "glRasterPos3fvFunc", egl_rasterPos3fv },
   { glRasterPos3ivFunc,  "glRasterPos3ivFunc", egl_rasterPos3iv },
   { glRasterPos3svFunc,  "glRasterPos3svFunc", egl_rasterPos3sv },
   { glRasterPos4dvFunc,  "glRasterPos4dvFunc", egl_rasterPos4dv },
   { glRasterPos4fvFunc,  "glRasterPos4fvFunc", egl_rasterPos4fv },
   { glRasterPos4ivFunc,  "glRasterPos4ivFunc", egl_rasterPos4iv },
   { glRasterPos4svFunc,  "glRasterPos4svFunc", egl_rasterPos4sv },
   { glReadBufferFunc,  "glReadBufferFunc", egl_readBuffer },
   { glReadPixelsFunc,  "glReadPixelsFunc", egl_readPixels },
   { glRectdvFunc,  "glRectdvFunc", egl_rectdv },
   { glRectfvFunc,  "glRectfvFunc", egl_rectfv },
   { glRectivFunc,  "glRectivFunc", egl_rectiv },
   { glRectsvFunc,  "glRectsvFunc", egl_rectsv },
   { glRenderModeFunc,  "glRenderModeFunc", egl_renderMode },
   { glRotatedFunc,  "glRotatedFunc", egl_rotated },
   { glRotatefFunc,  "glRotatefFunc", egl_rotatef },
   { glScaledFunc,  "glScaledFunc", egl_scaled },
   { glScalefFunc,  "glScalefFunc", egl_scalef },
   { glScissorFunc,  "glScissorFunc", egl_scissor },
   { glSelectBufferFunc,  "glSelectBufferFunc", egl_selectBuffer },
   { glShadeModelFunc,  "glShadeModelFunc", egl_shadeModel },
   { glStencilFuncFunc,  "glStencilFuncFunc", egl_stencilFunc },
   { glStencilMaskFunc,  "glStencilMaskFunc", egl_stencilMask },
   { glStencilOpFunc,  "glStencilOpFunc", egl_stencilOp },
   { glTexCoord1dvFunc,  "glTexCoord1dvFunc", egl_texCoord1dv },
   { glTexCoord1fvFunc,  "glTexCoord1fvFunc", egl_texCoord1fv },
   { glTexCoord1ivFunc,  "glTexCoord1ivFunc", egl_texCoord1iv },
   { glTexCoord1svFunc,  "glTexCoord1svFunc", egl_texCoord1sv },
   { glTexCoord2dvFunc,  "glTexCoord2dvFunc", egl_texCoord2dv },
   { glTexCoord2fvFunc,  "glTexCoord2fvFunc", egl_texCoord2fv },
   { glTexCoord2ivFunc,  "glTexCoord2ivFunc", egl_texCoord2iv },
   { glTexCoord2svFunc,  "glTexCoord2svFunc", egl_texCoord2sv },
   { glTexCoord3dvFunc,  "glTexCoord3dvFunc", egl_texCoord3dv },
   { glTexCoord3fvFunc,  "glTexCoord3fvFunc", egl_texCoord3fv },
   { glTexCoord3ivFunc,  "glTexCoord3ivFunc", egl_texCoord3iv },
   { glTexCoord3svFunc,  "glTexCoord3svFunc", egl_texCoord3sv },
   { glTexCoord4dvFunc,  "glTexCoord4dvFunc", egl_texCoord4dv },
   { glTexCoord4fvFunc,  "glTexCoord4fvFunc", egl_texCoord4fv },
   { glTexCoord4ivFunc,  "glTexCoord4ivFunc", egl_texCoord4iv },
   { glTexCoord4svFunc,  "glTexCoord4svFunc", egl_texCoord4sv },
   { glTexCoordPointerFunc,  "glTexCoordPointerFunc", egl_texCoordPointer },
   { glTexEnvfFunc,  "glTexEnvfFunc", egl_texEnvf },
   { glTexEnvfvFunc,  "glTexEnvfvFunc", egl_texEnvfv },
   { glTexEnviFunc,  "glTexEnviFunc", egl_texEnvi },
   { glTexEnvivFunc,  "glTexEnvivFunc", egl_texEnviv },
   { glTexGendFunc,  "glTexGendFunc", egl_texGend },
   { glTexGendvFunc,  "glTexGendvFunc", egl_texGendv },
   { glTexGenfFunc,  "glTexGenfFunc", egl_texGenf },
   { glTexGenfvFunc,  "glTexGenfvFunc", egl_texGenfv },
   { glTexGeniFunc,  "glTexGeniFunc", egl_texGeni },
   { glTexGenivFunc,  "glTexGenivFunc", egl_texGeniv },
   { glTexImage1DFunc,  "glTexImage1DFunc", egl_texImage1D },
   { glTexImage2DFunc,  "glTexImage2DFunc", egl_texImage2D },
   { glTexParameterfFunc,  "glTexParameterfFunc", egl_texParameterf },
   { glTexParameterfvFunc,  "glTexParameterfvFunc", egl_texParameterfv },
   { glTexParameteriFunc,  "glTexParameteriFunc", egl_texParameteri },
   { glTexParameterivFunc,  "glTexParameterivFunc", egl_texParameteriv },
   { glTexSubImage1DFunc,  "glTexSubImage1DFunc", egl_texSubImage1D },
   { glTexSubImage2DFunc,  "glTexSubImage2DFunc", egl_texSubImage2D },
   { glTranslatedFunc,  "glTranslatedFunc", egl_translated },
   { glTranslatefFunc,  "glTranslatefFunc", egl_translatef },
   { glVertex2dvFunc,  "glVertex2dvFunc", egl_vertex2dv },
   { glVertex2fvFunc,  "glVertex2fvFunc", egl_vertex2fv },
   { glVertex2ivFunc,  "glVertex2ivFunc", egl_vertex2iv },
   { glVertex2svFunc,  "glVertex2svFunc", egl_vertex2sv },
   { glVertex3dvFunc,  "glVertex3dvFunc", egl_vertex3dv },
   { glVertex3fvFunc,  "glVertex3fvFunc", egl_vertex3fv },
   { glVertex3ivFunc,  "glVertex3ivFunc", egl_vertex3iv },
   { glVertex3svFunc,  "glVertex3svFunc", egl_vertex3sv },
   { glVertex4dvFunc,  "glVertex4dvFunc", egl_vertex4dv },
   { glVertex4fvFunc,  "glVertex4fvFunc", egl_vertex4fv },
   { glVertex4ivFunc,  "glVertex4ivFunc", egl_vertex4iv },
   { glVertex4svFunc,  "glVertex4svFunc", egl_vertex4sv },
   { glVertexPointerFunc,  "glVertexPointerFunc", egl_vertexPointer },
   { glViewportFunc,  "glViewportFunc", egl_viewport },


   /*  MY GL memory handling funcs */
   { mygl_allocFunc, "mygl_allocFunc", mygl_alloc },
   { mygl_writeFunc, "mygl_writeFunc", mygl_write },

   /***** GLU FUNCTIONS *****/
   { gluBeginCurveFunc,  "gluBeginCurve", eglu_beginCurve },
   { gluBeginSurfaceFunc,  "gluBeginSurface", eglu_beginSurface },
   { gluBeginTrimFunc,  "gluBeginTrim", eglu_beginTrim },
   { gluBuild1DMipmapsFunc,  "gluBuild1DMipmaps", eglu_build1DMipmaps },
   { gluBuild2DMipmapsFunc,  "gluBuild2DMipmaps", eglu_build2DMipmaps },
   { gluCylinderFunc,  "gluCylinder", eglu_cylinder },
   { gluDeleteNurbsRendererFunc,  "gluDeleteNurbsRenderer", eglu_deleteNurbsRenderer },
   { gluDeleteQuadricFunc,  "gluDeleteQuadric", eglu_deleteQuadric },
   { gluDeleteTessFunc,  "gluDeleteTess", eglu_deleteTess },
   { gluDiskFunc,  "gluDisk", eglu_disk },
   { gluEndCurveFunc,  "gluEndCurve", eglu_endCurve },
   { gluEndSurfaceFunc,  "gluEndSurface", eglu_endSurface },
   { gluEndTrimFunc,  "gluEndTrim", eglu_endTrim },
   { gluErrorStringFunc,  "gluErrorString", eglu_errorString },
   { gluGetNurbsPropertyFunc,  "gluGetNurbsProperty", eglu_getNurbsProperty },
   { gluGetStringFunc,  "gluGetString", eglu_getString },
   { gluGetTessPropertyFunc,  "gluGetTessProperty", eglu_getTessProperty },
   { gluLoadSamplingMatricesFunc,  "gluLoadSamplingMatrices", eglu_loadSamplingMatrices },
   { gluLookAtFunc,  "gluLookAt", eglu_lookAt },
   { gluNewNurbsRendererFunc,  "gluNewNurbsRenderer", eglu_newNurbsRenderer },
   { gluNewQuadricFunc,  "gluNewQuadric", eglu_newQuadric },
   { gluNewTessFunc,  "gluNewTess", eglu_newTess },
   { gluNurbsCurveFunc,  "gluNurbsCurve", eglu_nurbsCurve },
   { gluNurbsPropertyFunc,  "gluNurbsProperty", eglu_nurbsProperty },
   { gluNurbsSurfaceFunc,  "gluNurbsSurface", eglu_nurbsSurface },
   { gluOrtho2DFunc,  "gluOrtho2D", eglu_ortho2D },
   { gluPartialDiskFunc,  "gluPartialDisk", eglu_partialDisk },
   { gluPerspectiveFunc,  "gluPerspective", eglu_perspective },
   { gluPickMatrixFunc,  "gluPickMatrix", eglu_pickMatrix },
   { gluProjectFunc,  "gluProject", eglu_project },
   { gluPwlCurveFunc,  "gluPwlCurve", eglu_pwlCurve },
   { gluQuadricDrawStyleFunc,  "gluQuadricDrawStyle", eglu_quadricDrawStyle },
   { gluQuadricNormalsFunc,  "gluQuadricNormals", eglu_quadricNormals },
   { gluQuadricOrientationFunc,  "gluQuadricOrientation", eglu_quadricOrientation },
   { gluQuadricTextureFunc,  "gluQuadricTexture", eglu_quadricTexture },
   { gluScaleImageFunc,  "gluScaleImage", eglu_scaleImage },
   { gluSphereFunc,  "gluSphere", eglu_sphere },
   { gluTessBeginContourFunc,  "gluTessBeginContour", eglu_tessBeginContour },
   { gluTessBeginPolygonFunc,  "gluTessBeginPolygon", eglu_tessBeginPolygon },
   { gluTessCallbackFunc,  "gluTessCallback", eglu_tessCallback },
   { gluTessEndContourFunc,  "gluTessEndContour", eglu_tessEndContour },
   { gluTessEndPolygonFunc,  "gluTessEndPolygon", eglu_tessEndPolygon },
   { gluTessNormalFunc,  "gluTessNormal", eglu_tessNormal },
   { gluTessPropertyFunc,  "gluTessProperty", eglu_tessProperty },
   { gluTessVertexFunc,  "gluTessVertex", eglu_tessVertex },
   { gluUnProjectFunc,  "gluUnProject", eglu_unProject },
   { 0, "LastFunction", NULL}};

static struct {
  int op;
  char* name;
  sdl_fun fun;
  void* ext_fun;
} ext_fns[] = {
  /*  Opengl 1.2 or later with ARB's and requested extensions  */
  { glBlendColorFunc,  "glBlendColor", egl_blendColor, &esdl_glBlendColor },
  { glBlendEquationFunc,  "glBlendEquation", egl_blendEquation, &esdl_glBlendEquation },
  { glDrawRangeElementsFunc,  "glDrawRangeElements", egl_drawRangeElements, &esdl_glDrawRangeElements },
  { glColorTableFunc,  "glColorTable", egl_colorTable, &esdl_glColorTable },
  { glColorTableParameterfvFunc,  "glColorTableParameterfv", egl_colorTableParameterfv, &esdl_glColorTableParameterfv },
  { glColorTableParameterivFunc,  "glColorTableParameteriv", egl_colorTableParameteriv, &esdl_glColorTableParameteriv },
  { glCopyColorTableFunc,  "glCopyColorTable", egl_copyColorTable, &esdl_glCopyColorTable },
  { glGetColorTableFunc,  "glGetColorTable", egl_getColorTable, &esdl_glGetColorTable },
  { glGetColorTableParameterfvFunc,  "glGetColorTableParameterfv", egl_getColorTableParameterfv, &esdl_glGetColorTableParameterfv },
  { glGetColorTableParameterivFunc,  "glGetColorTableParameteriv", egl_getColorTableParameteriv, &esdl_glGetColorTableParameteriv },
  { glColorSubTableFunc,  "glColorSubTable", egl_colorSubTable, &esdl_glColorSubTable },
  { glCopyColorSubTableFunc,  "glCopyColorSubTable", egl_copyColorSubTable, &esdl_glCopyColorSubTable },
  { glConvolutionFilter1DFunc,  "glConvolutionFilter1D", egl_convolutionFilter1D, &esdl_glConvolutionFilter1D },
  { glConvolutionFilter2DFunc,  "glConvolutionFilter2D", egl_convolutionFilter2D, &esdl_glConvolutionFilter2D },
  { glConvolutionParameterfFunc,  "glConvolutionParameterf", egl_convolutionParameterf, &esdl_glConvolutionParameterf },
  { glConvolutionParameterfvFunc,  "glConvolutionParameterfv", egl_convolutionParameterfv, &esdl_glConvolutionParameterfv },
  { glConvolutionParameteriFunc,  "glConvolutionParameteri", egl_convolutionParameteri, &esdl_glConvolutionParameteri },
  { glConvolutionParameterivFunc,  "glConvolutionParameteriv", egl_convolutionParameteriv, &esdl_glConvolutionParameteriv },
  { glCopyConvolutionFilter1DFunc,  "glCopyConvolutionFilter1D", egl_copyConvolutionFilter1D, &esdl_glCopyConvolutionFilter1D },
  { glCopyConvolutionFilter2DFunc,  "glCopyConvolutionFilter2D", egl_copyConvolutionFilter2D, &esdl_glCopyConvolutionFilter2D },
  { glGetConvolutionFilterFunc,  "glGetConvolutionFilter", egl_getConvolutionFilter, &esdl_glGetConvolutionFilter },
  { glGetConvolutionParameterfvFunc,  "glGetConvolutionParameterfv", egl_getConvolutionParameterfv, &esdl_glGetConvolutionParameterfv },
  { glGetConvolutionParameterivFunc,  "glGetConvolutionParameteriv", egl_getConvolutionParameteriv, &esdl_glGetConvolutionParameteriv },
  { glGetSeparableFilterFunc,  "glGetSeparableFilter", egl_getSeparableFilter, &esdl_glGetSeparableFilter },
  { glSeparableFilter2DFunc,  "glSeparableFilter2D", egl_separableFilter2D, &esdl_glSeparableFilter2D },
  { glGetHistogramFunc,  "glGetHistogram", egl_getHistogram, &esdl_glGetHistogram },
  { glGetHistogramParameterfvFunc,  "glGetHistogramParameterfv", egl_getHistogramParameterfv, &esdl_glGetHistogramParameterfv },
  { glGetHistogramParameterivFunc,  "glGetHistogramParameteriv", egl_getHistogramParameteriv, &esdl_glGetHistogramParameteriv },
  { glGetMinmaxFunc,  "glGetMinmax", egl_getMinmax, &esdl_glGetMinmax },
  { glGetMinmaxParameterfvFunc,  "glGetMinmaxParameterfv", egl_getMinmaxParameterfv, &esdl_glGetMinmaxParameterfv },
  { glGetMinmaxParameterivFunc,  "glGetMinmaxParameteriv", egl_getMinmaxParameteriv, &esdl_glGetMinmaxParameteriv },
  { glHistogramFunc,  "glHistogram", egl_histogram, &esdl_glHistogram },
  { glMinmaxFunc,  "glMinmax", egl_minmax, &esdl_glMinmax },
  { glResetHistogramFunc,  "glResetHistogram", egl_resetHistogram, &esdl_glResetHistogram },
  { glResetMinmaxFunc,  "glResetMinmax", egl_resetMinmax, &esdl_glResetMinmax },
  { glTexImage3DFunc,  "glTexImage3D", egl_texImage3D, &esdl_glTexImage3D },
  { glTexSubImage3DFunc,  "glTexSubImage3D", egl_texSubImage3D, &esdl_glTexSubImage3D },
  { glCopyTexSubImage3DFunc,  "glCopyTexSubImage3D", egl_copyTexSubImage3D, &esdl_glCopyTexSubImage3D },
  { glActiveTextureFunc,  "glActiveTexture", egl_activeTexture, &esdl_glActiveTexture },
  { glClientActiveTextureFunc,  "glClientActiveTexture", egl_clientActiveTexture, &esdl_glClientActiveTexture },
  { glMultiTexCoord1dvFunc,  "glMultiTexCoord1dv", egl_multiTexCoord1dv, &esdl_glMultiTexCoord1dv },
  { glMultiTexCoord1fvFunc,  "glMultiTexCoord1fv", egl_multiTexCoord1fv, &esdl_glMultiTexCoord1fv },
  { glMultiTexCoord1ivFunc,  "glMultiTexCoord1iv", egl_multiTexCoord1iv, &esdl_glMultiTexCoord1iv },
  { glMultiTexCoord1svFunc,  "glMultiTexCoord1sv", egl_multiTexCoord1sv, &esdl_glMultiTexCoord1sv },
  { glMultiTexCoord2dvFunc,  "glMultiTexCoord2dv", egl_multiTexCoord2dv, &esdl_glMultiTexCoord2dv },
  { glMultiTexCoord2fvFunc,  "glMultiTexCoord2fv", egl_multiTexCoord2fv, &esdl_glMultiTexCoord2fv },
  { glMultiTexCoord2ivFunc,  "glMultiTexCoord2iv", egl_multiTexCoord2iv, &esdl_glMultiTexCoord2iv },
  { glMultiTexCoord2svFunc,  "glMultiTexCoord2sv", egl_multiTexCoord2sv, &esdl_glMultiTexCoord2sv },
  { glMultiTexCoord3dvFunc,  "glMultiTexCoord3dv", egl_multiTexCoord3dv, &esdl_glMultiTexCoord3dv },
  { glMultiTexCoord3fvFunc,  "glMultiTexCoord3fv", egl_multiTexCoord3fv, &esdl_glMultiTexCoord3fv },
  { glMultiTexCoord3ivFunc,  "glMultiTexCoord3iv", egl_multiTexCoord3iv, &esdl_glMultiTexCoord3iv },
  { glMultiTexCoord3svFunc,  "glMultiTexCoord3sv", egl_multiTexCoord3sv, &esdl_glMultiTexCoord3sv },
  { glMultiTexCoord4dvFunc,  "glMultiTexCoord4dv", egl_multiTexCoord4dv, &esdl_glMultiTexCoord4dv },
  { glMultiTexCoord4fvFunc,  "glMultiTexCoord4fv", egl_multiTexCoord4fv, &esdl_glMultiTexCoord4fv },
  { glMultiTexCoord4ivFunc,  "glMultiTexCoord4iv", egl_multiTexCoord4iv, &esdl_glMultiTexCoord4iv },
  { glMultiTexCoord4svFunc,  "glMultiTexCoord4sv", egl_multiTexCoord4sv, &esdl_glMultiTexCoord4sv },
  { glLoadTransposeMatrixfFunc,  "glLoadTransposeMatrixf", egl_loadTransposeMatrixf, &esdl_glLoadTransposeMatrixf },
  { glLoadTransposeMatrixdFunc,  "glLoadTransposeMatrixd", egl_loadTransposeMatrixd, &esdl_glLoadTransposeMatrixd },
  { glMultTransposeMatrixfFunc,  "glMultTransposeMatrixf", egl_multTransposeMatrixf, &esdl_glMultTransposeMatrixf },
  { glMultTransposeMatrixdFunc,  "glMultTransposeMatrixd", egl_multTransposeMatrixd, &esdl_glMultTransposeMatrixd },
  { glSampleCoverageFunc,  "glSampleCoverage", egl_sampleCoverage, &esdl_glSampleCoverage },
  { glCompressedTexImage3DFunc,  "glCompressedTexImage3D", egl_compressedTexImage3D, &esdl_glCompressedTexImage3D },
  { glCompressedTexImage2DFunc,  "glCompressedTexImage2D", egl_compressedTexImage2D, &esdl_glCompressedTexImage2D },
  { glCompressedTexImage1DFunc,  "glCompressedTexImage1D", egl_compressedTexImage1D, &esdl_glCompressedTexImage1D },
  { glCompressedTexSubImage3DFunc,  "glCompressedTexSubImage3D", egl_compressedTexSubImage3D, &esdl_glCompressedTexSubImage3D },
  { glCompressedTexSubImage2DFunc,  "glCompressedTexSubImage2D", egl_compressedTexSubImage2D, &esdl_glCompressedTexSubImage2D },
  { glCompressedTexSubImage1DFunc,  "glCompressedTexSubImage1D", egl_compressedTexSubImage1D, &esdl_glCompressedTexSubImage1D },
  { glGetCompressedTexImageFunc,  "glGetCompressedTexImage", egl_getCompressedTexImage, &esdl_glGetCompressedTexImage },
  { glBlendFuncSeparateFunc,  "glBlendFuncSeparate", egl_blendFuncSeparate, &esdl_glBlendFuncSeparate },
  { glFogCoordfFunc,  "glFogCoordf", egl_fogCoordf, &esdl_glFogCoordf },
  { glFogCoordfvFunc,  "glFogCoordfv", egl_fogCoordfv, &esdl_glFogCoordfv },
  { glFogCoorddFunc,  "glFogCoordd", egl_fogCoordd, &esdl_glFogCoordd },
  { glFogCoorddvFunc,  "glFogCoorddv", egl_fogCoorddv, &esdl_glFogCoorddv },
  { glFogCoordPointerFunc,  "glFogCoordPointer", egl_fogCoordPointer, &esdl_glFogCoordPointer },
  { glMultiDrawArraysFunc,  "glMultiDrawArrays", egl_multiDrawArrays, &esdl_glMultiDrawArrays },
  { glPointParameterfFunc,  "glPointParameterf", egl_pointParameterf, &esdl_glPointParameterf },
  { glPointParameterfvFunc,  "glPointParameterfv", egl_pointParameterfv, &esdl_glPointParameterfv },
  { glPointParameteriFunc,  "glPointParameteri", egl_pointParameteri, &esdl_glPointParameteri },
  { glPointParameterivFunc,  "glPointParameteriv", egl_pointParameteriv, &esdl_glPointParameteriv },
  { glSecondaryColor3bvFunc,  "glSecondaryColor3bv", egl_secondaryColor3bv, &esdl_glSecondaryColor3bv },
  { glSecondaryColor3dvFunc,  "glSecondaryColor3dv", egl_secondaryColor3dv, &esdl_glSecondaryColor3dv },
  { glSecondaryColor3fvFunc,  "glSecondaryColor3fv", egl_secondaryColor3fv, &esdl_glSecondaryColor3fv },
  { glSecondaryColor3ivFunc,  "glSecondaryColor3iv", egl_secondaryColor3iv, &esdl_glSecondaryColor3iv },
  { glSecondaryColor3svFunc,  "glSecondaryColor3sv", egl_secondaryColor3sv, &esdl_glSecondaryColor3sv },
  { glSecondaryColor3ubvFunc,  "glSecondaryColor3ubv", egl_secondaryColor3ubv, &esdl_glSecondaryColor3ubv },
  { glSecondaryColor3uivFunc,  "glSecondaryColor3uiv", egl_secondaryColor3uiv, &esdl_glSecondaryColor3uiv },
  { glSecondaryColor3usvFunc,  "glSecondaryColor3usv", egl_secondaryColor3usv, &esdl_glSecondaryColor3usv },
  { glSecondaryColorPointerFunc,  "glSecondaryColorPointer", egl_secondaryColorPointer, &esdl_glSecondaryColorPointer },
  { glWindowPos2dvFunc,  "glWindowPos2dv", egl_windowPos2dv, &esdl_glWindowPos2dv },
  { glWindowPos2fvFunc,  "glWindowPos2fv", egl_windowPos2fv, &esdl_glWindowPos2fv },
  { glWindowPos2ivFunc,  "glWindowPos2iv", egl_windowPos2iv, &esdl_glWindowPos2iv },
  { glWindowPos2svFunc,  "glWindowPos2sv", egl_windowPos2sv, &esdl_glWindowPos2sv },
  { glWindowPos3dvFunc,  "glWindowPos3dv", egl_windowPos3dv, &esdl_glWindowPos3dv },
  { glWindowPos3fvFunc,  "glWindowPos3fv", egl_windowPos3fv, &esdl_glWindowPos3fv },
  { glWindowPos3ivFunc,  "glWindowPos3iv", egl_windowPos3iv, &esdl_glWindowPos3iv },
  { glWindowPos3svFunc,  "glWindowPos3sv", egl_windowPos3sv, &esdl_glWindowPos3sv },
  { glWeightbvARBFunc,  "glWeightbvARB", egl_weightbvARB, &esdl_glWeightbvARB },
  { glWeightsvARBFunc,  "glWeightsvARB", egl_weightsvARB, &esdl_glWeightsvARB },
  { glWeightivARBFunc,  "glWeightivARB", egl_weightivARB, &esdl_glWeightivARB },
  { glWeightfvARBFunc,  "glWeightfvARB", egl_weightfvARB, &esdl_glWeightfvARB },
  { glWeightdvARBFunc,  "glWeightdvARB", egl_weightdvARB, &esdl_glWeightdvARB },
  { glWeightubvARBFunc,  "glWeightubvARB", egl_weightubvARB, &esdl_glWeightubvARB },
  { glWeightusvARBFunc,  "glWeightusvARB", egl_weightusvARB, &esdl_glWeightusvARB },
  { glWeightuivARBFunc,  "glWeightuivARB", egl_weightuivARB, &esdl_glWeightuivARB },
  { glWeightPointerARBFunc,  "glWeightPointerARB", egl_weightPointerARB, &esdl_glWeightPointerARB },
  { glVertexBlendARBFunc,  "glVertexBlendARB", egl_vertexBlendARB, &esdl_glVertexBlendARB },
  { glCurrentPaletteMatrixARBFunc,  "glCurrentPaletteMatrixARB", egl_currentPaletteMatrixARB, &esdl_glCurrentPaletteMatrixARB },
  { glMatrixIndexubvARBFunc,  "glMatrixIndexubvARB", egl_matrixIndexubvARB, &esdl_glMatrixIndexubvARB },
  { glMatrixIndexusvARBFunc,  "glMatrixIndexusvARB", egl_matrixIndexusvARB, &esdl_glMatrixIndexusvARB },
  { glMatrixIndexuivARBFunc,  "glMatrixIndexuivARB", egl_matrixIndexuivARB, &esdl_glMatrixIndexuivARB },
  { glMatrixIndexPointerARBFunc,  "glMatrixIndexPointerARB", egl_matrixIndexPointerARB, &esdl_glMatrixIndexPointerARB },
  { glVertexAttrib1dvARBFunc,  "glVertexAttrib1dvARB", egl_vertexAttrib1dvARB, &esdl_glVertexAttrib1dvARB },
  { glVertexAttrib1fvARBFunc,  "glVertexAttrib1fvARB", egl_vertexAttrib1fvARB, &esdl_glVertexAttrib1fvARB },
  { glVertexAttrib1svARBFunc,  "glVertexAttrib1svARB", egl_vertexAttrib1svARB, &esdl_glVertexAttrib1svARB },
  { glVertexAttrib2dvARBFunc,  "glVertexAttrib2dvARB", egl_vertexAttrib2dvARB, &esdl_glVertexAttrib2dvARB },
  { glVertexAttrib2fvARBFunc,  "glVertexAttrib2fvARB", egl_vertexAttrib2fvARB, &esdl_glVertexAttrib2fvARB },
  { glVertexAttrib2svARBFunc,  "glVertexAttrib2svARB", egl_vertexAttrib2svARB, &esdl_glVertexAttrib2svARB },
  { glVertexAttrib3dvARBFunc,  "glVertexAttrib3dvARB", egl_vertexAttrib3dvARB, &esdl_glVertexAttrib3dvARB },
  { glVertexAttrib3fvARBFunc,  "glVertexAttrib3fvARB", egl_vertexAttrib3fvARB, &esdl_glVertexAttrib3fvARB },
  { glVertexAttrib3svARBFunc,  "glVertexAttrib3svARB", egl_vertexAttrib3svARB, &esdl_glVertexAttrib3svARB },
  { glVertexAttrib4NbvARBFunc,  "glVertexAttrib4NbvARB", egl_vertexAttrib4NbvARB, &esdl_glVertexAttrib4NbvARB },
  { glVertexAttrib4NivARBFunc,  "glVertexAttrib4NivARB", egl_vertexAttrib4NivARB, &esdl_glVertexAttrib4NivARB },
  { glVertexAttrib4NsvARBFunc,  "glVertexAttrib4NsvARB", egl_vertexAttrib4NsvARB, &esdl_glVertexAttrib4NsvARB },
  { glVertexAttrib4NubARBFunc,  "glVertexAttrib4NubARB", egl_vertexAttrib4NubARB, &esdl_glVertexAttrib4NubARB },
  { glVertexAttrib4NubvARBFunc,  "glVertexAttrib4NubvARB", egl_vertexAttrib4NubvARB, &esdl_glVertexAttrib4NubvARB },
  { glVertexAttrib4NuivARBFunc,  "glVertexAttrib4NuivARB", egl_vertexAttrib4NuivARB, &esdl_glVertexAttrib4NuivARB },
  { glVertexAttrib4NusvARBFunc,  "glVertexAttrib4NusvARB", egl_vertexAttrib4NusvARB, &esdl_glVertexAttrib4NusvARB },
  { glVertexAttrib4bvARBFunc,  "glVertexAttrib4bvARB", egl_vertexAttrib4bvARB, &esdl_glVertexAttrib4bvARB },
  { glVertexAttrib4dvARBFunc,  "glVertexAttrib4dvARB", egl_vertexAttrib4dvARB, &esdl_glVertexAttrib4dvARB },
  { glVertexAttrib4fvARBFunc,  "glVertexAttrib4fvARB", egl_vertexAttrib4fvARB, &esdl_glVertexAttrib4fvARB },
  { glVertexAttrib4ivARBFunc,  "glVertexAttrib4ivARB", egl_vertexAttrib4ivARB, &esdl_glVertexAttrib4ivARB },
  { glVertexAttrib4svARBFunc,  "glVertexAttrib4svARB", egl_vertexAttrib4svARB, &esdl_glVertexAttrib4svARB },
  { glVertexAttrib4ubvARBFunc,  "glVertexAttrib4ubvARB", egl_vertexAttrib4ubvARB, &esdl_glVertexAttrib4ubvARB },
  { glVertexAttrib4uivARBFunc,  "glVertexAttrib4uivARB", egl_vertexAttrib4uivARB, &esdl_glVertexAttrib4uivARB },
  { glVertexAttrib4usvARBFunc,  "glVertexAttrib4usvARB", egl_vertexAttrib4usvARB, &esdl_glVertexAttrib4usvARB },
  { glVertexAttribPointerARBFunc,  "glVertexAttribPointerARB", egl_vertexAttribPointerARB, &esdl_glVertexAttribPointerARB },
  { glEnableVertexAttribArrayARBFunc,  "glEnableVertexAttribArrayARB", egl_enableVertexAttribArrayARB, &esdl_glEnableVertexAttribArrayARB },
  { glDisableVertexAttribArrayARBFunc,  "glDisableVertexAttribArrayARB", egl_disableVertexAttribArrayARB, &esdl_glDisableVertexAttribArrayARB },
  { glProgramStringARBFunc,  "glProgramStringARB", egl_programStringARB, &esdl_glProgramStringARB },
  { glBindProgramARBFunc,  "glBindProgramARB", egl_bindProgramARB, &esdl_glBindProgramARB },
  { glDeleteProgramsARBFunc,  "glDeleteProgramsARB", egl_deleteProgramsARB, &esdl_glDeleteProgramsARB },
  { glGenProgramsARBFunc,  "glGenProgramsARB", egl_genProgramsARB, &esdl_glGenProgramsARB },
  { glProgramEnvParameter4dvARBFunc,  "glProgramEnvParameter4dvARB", egl_programEnvParameter4dvARB, &esdl_glProgramEnvParameter4dvARB },
  { glProgramEnvParameter4fvARBFunc,  "glProgramEnvParameter4fvARB", egl_programEnvParameter4fvARB, &esdl_glProgramEnvParameter4fvARB },
  { glProgramLocalParameter4dvARBFunc,  "glProgramLocalParameter4dvARB", egl_programLocalParameter4dvARB, &esdl_glProgramLocalParameter4dvARB },
  { glProgramLocalParameter4fvARBFunc,  "glProgramLocalParameter4fvARB", egl_programLocalParameter4fvARB, &esdl_glProgramLocalParameter4fvARB },
  { glGetProgramEnvParameterdvARBFunc,  "glGetProgramEnvParameterdvARB", egl_getProgramEnvParameterdvARB, &esdl_glGetProgramEnvParameterdvARB },
  { glGetProgramEnvParameterfvARBFunc,  "glGetProgramEnvParameterfvARB", egl_getProgramEnvParameterfvARB, &esdl_glGetProgramEnvParameterfvARB },
  { glGetProgramLocalParameterdvARBFunc,  "glGetProgramLocalParameterdvARB", egl_getProgramLocalParameterdvARB, &esdl_glGetProgramLocalParameterdvARB },
  { glGetProgramLocalParameterfvARBFunc,  "glGetProgramLocalParameterfvARB", egl_getProgramLocalParameterfvARB, &esdl_glGetProgramLocalParameterfvARB },
  { glGetProgramivARBFunc,  "glGetProgramivARB", egl_getProgramivARB, &esdl_glGetProgramivARB },
  { glGetProgramStringARBFunc,  "glGetProgramStringARB", egl_getProgramStringARB, &esdl_glGetProgramStringARB },
  { glGetVertexAttribdvARBFunc,  "glGetVertexAttribdvARB", egl_getVertexAttribdvARB, &esdl_glGetVertexAttribdvARB },
  { glGetVertexAttribfvARBFunc,  "glGetVertexAttribfvARB", egl_getVertexAttribfvARB, &esdl_glGetVertexAttribfvARB },
  { glGetVertexAttribivARBFunc,  "glGetVertexAttribivARB", egl_getVertexAttribivARB, &esdl_glGetVertexAttribivARB },
  { glGetVertexAttribPointervARBFunc,  "glGetVertexAttribPointervARB", egl_getVertexAttribPointervARB, &esdl_glGetVertexAttribPointervARB },
  { glIsProgramARBFunc,  "glIsProgramARB", egl_isProgramARB, &esdl_glIsProgramARB },

  { 0, "LastFunction", NULL, NULL}};

static void
undefined_function(sdl_data *sd, int len, char *buff)
{
  sd->len = -2;
}

static void
undefined_extension(sdl_data *sd, int len, char *buff)
{
  sd->len = -3;
}

void init_fps(sdl_data* sd)
{   
  int i;
  int n = MAX_FUNCTIONS_H;  /* Must be greater than then the last function op */ 
  int op;
  sdl_fun* fun_tab;
  char** str_tab;

  fun_tab = sd->fun_tab = malloc((n+1) * sizeof(sdl_fun));
  str_tab = sd->str_tab = malloc((n+1) * sizeof(char*));

  for (i = 0; i < OPENGL_EXTS_H; i++) {
    fun_tab[i] = undefined_function;
    str_tab[i] = "unknown function";
  }

  for ( ; i < n; i++) {
    fun_tab[i] = undefined_extension;
    str_tab[i] = "unknown extension";
  }

  for (i = 0; ((op = code_fns[i].op) != 0); i++) {
    if (fun_tab[op] == undefined_function) {
      fun_tab[op] = code_fns[i].fn;
      str_tab[op] = code_fns[i].str;
    } else {
      fprintf(stderr, 
	      "FParray mismatch in initialization: %d '%s' %d '%s'\r\n",
	      i, str_tab[op], op, code_fns[i].str);
    }
  }
}

/* Must be done after creating a rendering context */
void init_glexts(sdl_data* sd)
{
  static int already_done = 0;

  if (already_done == 0) {
    int i;
    int op;
    sdl_fun* fun_tab = sd->fun_tab;
    char** str_tab = sd->str_tab;

    already_done = 1;
    for (i = 0; (op = ext_fns[i].op) != 0; i++) {
      if (fun_tab[op] == undefined_extension) {
	void* ext_ptr = SDL_GL_GetProcAddress(ext_fns[i].name);

	str_tab[op] = ext_fns[i].name;
	if (ext_ptr) {
	  /* fprintf(stderr, "Success %s \r\n", code_fns[i].str);*/
	  * (void **) (ext_fns[i].ext_fun) = ext_ptr;
	  fun_tab[op] = ext_fns[i].fun;
	} else {
	  /* fprintf(stderr, "Failed %s \r\n", code_fns[i].str); */
	  fun_tab[op] = undefined_extension;
	}
      } else {	 
	fprintf(stderr, "Exiting FP EXTENSION array mismatch in initialization %d %d %s\r\n", 
		i, ext_fns[i].op, ext_fns[i].name);
      }
    }
  }
}
